==> spring-boot-ext-1.2.279.7-sources.jar: cn/com/duibaboot/ext/autoconfigure/flowreplay/FlowReplayTrace.java <==
package cn.com.duibaboot.ext.autoconfigure.flowreplay;

import cn.com.duiba.wolf.utils.UUIDUtils;
import com.alibaba.ttl.TransmittableThreadLocal;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

/**
 * 流量录制用于回归的trace，一个trace表示一次调用的整条链路
 * Created by guoyanfei .
 * 2019-01-24 .
 */
@Slf4j
@Data
public class FlowReplayTrace implements Serializable {

    private static final long serialVersionUID = -622886226087802579L;

    /**
     * trace的唯一标识
     */
    private String traceId;

    private FlowReplaySpan mainSpan;

    private List<FlowReplaySpan> subSpans;

    private static class TransmittableThreadLocalHolder {

        private TransmittableThreadLocalHolder() {
        }

        private static final TransmittableThreadLocal<FlowReplayTrace> threadLocal2FlowReplayTrace = new TransmittableThreadLocal<>();
    }

    /**
     * 通过主调用创建trace
     * @param mainSpan
     */
    public static synchronized void createTrace(FlowReplaySpan mainSpan) {
        if (mainSpan == null) {
            return ;
        }
        FlowReplayTrace c = new FlowReplayTrace();
        c.traceId = UUIDUtils.createUUID();
        c.mainSpan = mainSpan;
        c.subSpans = new LinkedList<>();
        TransmittableThreadLocalHolder.threadLocal2FlowReplayTrace.set(c);
    }

    /**
     * 新增子调用
     * @param subSpan
     */
    public static synchronized void addSubSpan(FlowReplaySpan subSpan) {
        FlowReplayTrace trace = TransmittableThreadLocalHolder.threadLocal2FlowReplayTrace.get();
        if (trace == null || trace.subSpans == null) {
            return;
        }
        trace.subSpans.add(subSpan);
    }

    /**
     * 获取当前的traceId
     * @return
     */
    public static String getCurrentTraceId() {
        FlowReplayTrace trace = TransmittableThreadLocalHolder.threadLocal2FlowReplayTrace.get();
        if (trace == null) {
            return null;
        }
        return trace.traceId;
    }

    /**
     * 当前请求是否被追踪
     * @return
     */
    public static boolean isTraced() {
        return get() != null;
    }

    /**
     * 获取当前trace
     * @return
     */
    public static FlowReplayTrace get() {
        FlowReplayTrace trace = TransmittableThreadLocalHolder.threadLocal2FlowReplayTrace.get();
        if (trace == null) {
            return null;
        }
        return trace;
    }

    /**
     * 一般在异常的时候，才会直接调用这个
     */
    public static void remove() {
        TransmittableThreadLocalHolder.threadLocal2FlowReplayTrace.remove();
    }
}

==> spring-boot-ext-1.2.280.12-sources.jar: cn/com/duibaboot/ext/autoconfigure/flowreplay/FlowReplayTrace.java <==
package cn.com.duibaboot.ext.autoconfigure.flowreplay;

import cn.com.duiba.wolf.utils.UUIDUtils;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.record.RecordContextHolder;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.io.Serializable;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

/**
 * 流量录制用于回归的trace，一个trace表示一次调用的整条链路
 * Created by guoyanfei .
 * 2019-01-24 .
 */
@Slf4j
@Data
public class FlowReplayTrace implements Serializable {

    private static final long serialVersionUID = -622886226087802579L;

    /**
     * trace的唯一标识
     */
    private String traceId;

    private FlowReplaySpan mainSpan;

    private List<FlowReplaySpan> subSpans;

    private static class ThreadLocalHolder {

        private ThreadLocalHolder() {
        }

        private static final ThreadLocal<FlowReplayTrace> threadLocal2FlowReplayTrace = new ThreadLocal<>();
    }

    /**
     * 通过主调用创建trace
     * @param mainSpan
     */
    public static synchronized void createTrace(FlowReplaySpan mainSpan) {
        if (mainSpan == null) {
            return ;
        }
        FlowReplayTrace c = new FlowReplayTrace();
        c.traceId = UUIDUtils.createUUID();
        c.mainSpan = mainSpan;
        c.subSpans = Collections.synchronizedList(new LinkedList<>());
        ThreadLocalHolder.threadLocal2FlowReplayTrace.set(c);
    }

    /**
     * 新增子调用
     * @param subSpan
     */
    public static synchronized void addSubSpan(FlowReplaySpan subSpan) {
        FlowReplayTrace trace = ThreadLocalHolder.threadLocal2FlowReplayTrace.get();
        if (trace == null || trace.subSpans == null) {
            return;
        }
        trace.subSpans.add(subSpan);
    }

    /**
     * 获取当前的traceId
     * @return
     */
    public static String getCurrentTraceId() {
        FlowReplayTrace trace = ThreadLocalHolder.threadLocal2FlowReplayTrace.get();
        if (trace == null) {
            return null;
        }
        return trace.traceId;
    }

    /**
     * 当前请求是否被追踪
     * @return
     */
    public static boolean isTraced() {
        return RecordContextHolder.isRecording() && get() != null;
    }

    /**
     * 获取当前trace
     * @return
     */
    public static FlowReplayTrace get() {
        FlowReplayTrace trace = ThreadLocalHolder.threadLocal2FlowReplayTrace.get();
        if (trace == null) {
            return null;
        }
        return trace;
    }

    /**
     * 一般在异常的时候，才会直接调用这个
     */
    public static void remove() {
        ThreadLocalHolder.threadLocal2FlowReplayTrace.remove();
    }
}
